package de.jpv.accessors;

import com.intellij.codeInsight.intention.IntentionAction;
import com.intellij.psi.JavaResolveResult;
import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiClassType;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiExpression;
import com.intellij.psi.PsiReferenceExpression;
import com.intellij.psi.PsiType;
import com.intellij.psi.PsiVariable;
import com.intellij.psi.PsiAssignmentExpression;
import com.intellij.psi.infos.ClassCandidateInfo;
import com.intellij.psi.infos.MethodCandidateInfo;

public abstract class AccessorAction implements IntentionAction {

  protected static AccessorContext findAccessorContext(PsiElement element) {
    if (!(element instanceof PsiReferenceExpression)) {
      return null;
    }
    PsiReferenceExpression elementReference = (PsiReferenceExpression) element;
    JavaResolveResult elementResult = elementReference.advancedResolve(false);
    if (elementResult instanceof ClassCandidateInfo
        || elementResult instanceof MethodCandidateInfo) {
      return null;
    }
    PsiExpression qualifier = elementReference.getQualifierExpression();
    if (qualifier == null || !(qualifier instanceof PsiReferenceExpression)) {
      return null;
    }
    PsiReferenceExpression qualifierReference = (PsiReferenceExpression) qualifier;
    JavaResolveResult qualifierResult = qualifierReference.advancedResolve(false);
    PsiElement qualifierElement = qualifierResult.getElement();
    if (qualifierElement == null) {
      return null;
    }
    PsiClass psiClass = null;
    if (qualifierElement instanceof PsiVariable) {
      PsiType t = ((PsiVariable) qualifierElement).getType();
      if (!(t instanceof PsiClassType)) {
        return null;
      }
      psiClass = ((PsiClassType) t).resolve();
    }
    if (qualifierElement instanceof PsiClass) {
      psiClass = (PsiClass) qualifierElement;
    }
    if (psiClass == null) {
      return null;
    }
    if (elementReference.getParent() instanceof PsiAssignmentExpression) {
      PsiAssignmentExpression assignment = (PsiAssignmentExpression) elementReference.getParent();
      if (assignment.getLExpression().equals(elementReference)) {
        return new AccessorContext(elementReference, psiClass, AccessorContext.Type.SETTER, null, null);
      }
    }
    return new AccessorContext(elementReference, psiClass, AccessorContext.Type.GETTER, null, null);
  }

  public boolean startInWriteAction() {
    return false;
  }
}
