package de.jpv.accessors;

import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.PsiMethod;
import com.intellij.psi.PsiAssignmentExpression;
import com.intellij.util.IncorrectOperationException;

public class GetterAction extends AccessorAction {

  public static AccessorContext findGetterContext(PsiElement element) {
    AccessorContext retVal = findAccessorContext(element);
    if (retVal == null || retVal.getAccessorType() == AccessorContext.Type.SETTER) {
      return null;
    }
    String referenceName = retVal.getReference().getReferenceName();
    for (String prefix : new String[] {"get", "is"}) {
      String methodName = prefix + Character.toUpperCase(referenceName.charAt(0)) + referenceName.substring(1);
      PsiMethod[] methods = retVal.getPsiClass().findMethodsByName(methodName, true);
      for (PsiMethod method : methods) {
        if (method.getParameterList().getParameters().length == 0
            && method.getModifierList().hasModifierProperty("public")) {
          if ("get".equals(prefix)
              || (method.getReturnType() != null
                  && "boolean".equals(method.getReturnType().getCanonicalText()))) {
            retVal.setGetterMethod(method);
            return retVal;
          }
        }
      }
    }
    return null;
  }

  private static AccessorContext findGetter(Editor editor, PsiFile psiFile) {
    CaretModel caretModel = editor.getCaretModel();
    int position = caretModel.getOffset();
    for (PsiElement element = psiFile.findElementAt(position);
         element != null; element = element.getParent()) {
      AccessorContext ctx = findGetterContext(element);
      if (ctx != null) {
        return ctx;
      }
    }
    return null;
  }

  public String getFamilyName() {
    return "getterAction";
  }

  public String getText() {
    return "Replace with getter method";
  }

  public boolean isAvailable(Project project, Editor editor, PsiFile psiFile) {
    return findGetter(editor, psiFile) != null;
  }

  public void invoke(Project project, Editor editor, PsiFile psiFile)
          throws IncorrectOperationException {
    AccessorContext ctx = findGetter(editor, psiFile);
    // At this point it would be very nice to simply construct an instance of
    // PsiMethodCallExpression, but the OpenAPI doesn't seem to support this!
    ctx.getReference().getReferenceNameElement().delete();
    CaretModel caretModel = editor.getCaretModel();
    int offset = caretModel.getOffset();
    Document document = editor.getDocument();
    document.insertString(offset, ctx.getGetterMethod().getName() + "()");
  }

}
