package de.jpv.accessors;

import java.util.Arrays;

import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiMethod;
import com.intellij.psi.PsiFile;
import com.intellij.psi.PsiAssignmentExpression;
import com.intellij.psi.PsiType;
import com.intellij.psi.PsiExpression;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.project.Project;
import com.intellij.util.IncorrectOperationException;

public class SetterAction extends AccessorAction {

  private static boolean isSubtypeOf(PsiType subType, PsiType superType) {
    if (subType.equals(superType)) {
      return true;
    }
    for (PsiType type : subType.getSuperTypes()) {
      if (type.equals(superType)) {
        return true;
      }
    }
    return false;
  }

  public static AccessorContext findSetterContext(PsiElement element) {
    AccessorContext retVal = findAccessorContext(element);
    if (retVal == null || retVal.getAccessorType() == AccessorContext.Type.GETTER) {
      return null;
    }
    PsiAssignmentExpression ass = ((PsiAssignmentExpression) retVal.getReference().getParent());
    PsiType assignedObjectType = ass.getRExpression().getType();
    String referenceName = retVal.getReference().getReferenceName();
    String methodName = "set" + Character.toUpperCase(referenceName.charAt(0)) + referenceName.substring(1);
    PsiMethod[] methods = retVal.getPsiClass().findMethodsByName(methodName, true);
    for (PsiMethod method : methods) {
      if (method.getParameterList().getParameters().length == 1
          && method.getReturnType().getCanonicalText().equals("void")
          && method.getModifierList().hasModifierProperty("public")
          && isSubtypeOf(assignedObjectType,
                         method.getParameterList().getParameters()[0].getType())) {
        retVal.setSetterMethod(method);
        return retVal;
      }
    }
    return null;
  }

  private static AccessorContext findSetter(Editor editor, PsiFile psiFile) {
    CaretModel caretModel = editor.getCaretModel();
    int position = caretModel.getOffset();
    for (PsiElement element = psiFile.findElementAt(position);
         element != null; element = element.getParent()) {
      AccessorContext ctx = SetterAction.findSetterContext(element);
      if (ctx != null) {
        return ctx;
      }
    }
    return null;
  }

  public String getFamilyName() {
    return "setterAction";
  }

  public String getText() {
    return "Replace with setter method";
  }

  public boolean isAvailable(Project project, Editor editor, PsiFile psiFile) {
    return SetterAction.findSetter(editor, psiFile) != null;
  }

  public void invoke(Project project, Editor editor, PsiFile psiFile)
          throws IncorrectOperationException {
    AccessorContext ctx = SetterAction.findSetter(editor, psiFile);

    StringBuffer newText = new StringBuffer();
    newText.append(ctx.getReference().getQualifierExpression().getText());
    newText.append('.').append(ctx.getSetterMethod().getName()).append('(');

    PsiAssignmentExpression ass = ((PsiAssignmentExpression) ctx.getReference().getParent());
    newText.append(ass.getRExpression().getText()).append(')');
    ass.delete();
    CaretModel caretModel = editor.getCaretModel();
    Document document = editor.getDocument();
    document.insertString(caretModel.getOffset(), newText.toString());
  }

}
